import subprocess as sp
import multiprocessing as mp
from finntk.omor.anlys import pairs_to_dict


def analysis_to_pairs(ana):
    for bit in ana.split("]|["):
        k, v = bit.strip("[]").split("=", 1)
        yield k, v


def parse_finnpos_line(line):
    surf, _, lemma, feats, _ = line.split("\t")
    return surf, lemma, feats


def analysis_to_dict(ana):
    return pairs_to_dict(analysis_to_pairs(ana))


def batch_finnpos(source_iter, *args, maxsize=0, **kwargs):
    """
    Shovel sentences through FinnPOS. A typical use case for this would be when
    you want to tag a bunch of sentences with FinnPOS at the same time as doing
    some other kind of transformation.

    `source_iter` should be an iterator returning pairs `(sent, extra)` where
    sent is a list of tokens and `extra` is any sentence identifier you would
    like to pass through. It will be run in a new process. If you would like to
    pass arguments to it, pass them in as extra arguments to `batch_finnpos`.
    """
    done_sentinel = object()

    def source_func(pipe, queue, *args, **kwargs):
        for sent, extra in source_iter(*args, **kwargs):
            for token in sent:
                pipe.write(token)
                pipe.write("\n")
            pipe.write("\n")
            queue.put(extra)
        queue.put(done_sentinel)

    finnpos_proc = sp.Popen(
        ["ftb-label"], stdin=sp.PIPE, stdout=sp.PIPE, universal_newlines=True
    )
    ctx = mp.get_context("fork")
    id_queue = ctx.Queue(maxsize=0)
    source_proc = ctx.Process(
        target=source_func, args=(finnpos_proc.stdin, id_queue) + args, kwargs=kwargs
    )
    source_proc.start()
    while 1:
        extra = id_queue.get()
        if extra is done_sentinel:
            break
        tagged_sent = []
        for line in finnpos_proc.stdout:
            surf, lemma, feats_str = line[:-1]
            feats = analysis_to_dict(feats_str)
            tagged_sent.append((surf, lemma, feats))
        yield tagged_sent, extra
    source_proc.join()



import setuptools

setuptools.setup(
    name="finntk",
    version="0.0.9",
    url="https://github.com/frankier/finntk",
    author="Frankie Robertson",
    description="Finnish NLP toolkit",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    install_requires=[
        "more_itertools>=4.1.0",
        "pyahocorasick>=1.1.8",
        "appdirs>=1.4.3",
        "plumbum>=1.6.6",
        "nltk>=3.3",
        "gensim>=3.4.0",
    ],
    extras_require={
        "docs": [
            "sphinx_autodoc_typehints",
            "sphinx>=1.5",
            "sphinx_rtd_theme",
            "pytest_check_links",
            "recommonmark",
        ],
        "dev": [
            "pytest",
            # Markdown descriptions
            "twine>=1.11.0",
            "wheel>=0.31.0",
            "setuptools>=38.6.0",
        ],
    },
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
    ],
)
